@@ -54,7 +54,7 @@ static int calculate_fifo_occupancy(struct bnxt_re_context *cntx)
 	struct bnxt_re_dev *rdev = cntx->rdev;
 	uint32_t read_val, fifo_occup;
 	uint64_t fifo_reg_off;
-	uint64_t *dbr_map;
+	uint32_t *dbr_map;
 
 	fifo_reg_off =  pacing_data->grc_reg_offset & ~(BNXT_RE_PAGE_MASK(rdev->pg_size));
 	dbr_map = cntx->bar_map + fifo_reg_off;
@@ -165,12 +165,33 @@ void bnxt_re_ring_srq_arm(struct bnxt_re_srq *srq)
 	bnxt_re_ring_db(srq->udpi, &hdr);
 }
 
+/*
+ * During CQ resize, it is expected that the epoch needs to be maintained when
+ * switching from the old CQ to the new resized CQ.
+ *
+ * On the first CQ DB excecuted on the new CQ, we need to check if the index we
+ * are writing is less than the last index written for the old CQ. If that is
+ * the case, we need to flip the epoch so the ASIC does not get confused and
+ * think the CQ DB is out of order and therefore drop the DB (note the logic
+ * in the ASIC that checks CQ DB ordering is not aware of the CQ resize).
+ */
+static void bnxt_re_cq_resize_check(struct bnxt_re_queue *cqq)
+{
+	if (unlikely(cqq->cq_resized)) {
+		if (cqq->head < cqq->old_head)
+			cqq->flags ^= 1UL << BNXT_RE_FLAG_EPOCH_HEAD_SHIFT;
+
+		cqq->cq_resized = false;
+	}
+}
+
 void bnxt_re_ring_cq_db(struct bnxt_re_cq *cq)
 {
 	struct bnxt_re_db_hdr hdr;
 	uint32_t epoch;
 
 	bnxt_re_do_pacing(cq->cntx, &cq->rand);
+	bnxt_re_cq_resize_check(cq->cqq);
 	epoch = (cq->cqq->flags & BNXT_RE_FLAG_EPOCH_HEAD_MASK) << BNXT_RE_DB_EPOCH_HEAD_SHIFT;
 	bnxt_re_init_db_hdr(&hdr, cq->cqq->head | epoch, cq->cqid, 0, BNXT_RE_QUE_TYPE_CQ);
 	bnxt_re_ring_db(cq->udpi, &hdr);
@@ -191,6 +212,7 @@ void bnxt_re_ring_cq_arm_db(struct bnxt_re_cq *cq, uint8_t aflag)
 	}
 
 	bnxt_re_do_pacing(cq->cntx, &cq->rand);
+	bnxt_re_cq_resize_check(cq->cqq);
 	epoch = (cq->cqq->flags & BNXT_RE_FLAG_EPOCH_HEAD_MASK) <<  BNXT_RE_DB_EPOCH_HEAD_SHIFT;
 	bnxt_re_init_db_hdr(&hdr, cq->cqq->head | epoch, cq->cqid, toggle, aflag);
 	bnxt_re_ring_db(cq->udpi, &hdr);
