@@ -172,6 +172,9 @@ int efadv_query_device(struct ibv_context *ibvctx,
 
 		if (EFA_DEV_CAP(ctx, CQ_WITH_SGID))
 			attr->device_caps |= EFADV_DEVICE_ATTR_CAPS_CQ_WITH_SGID;
+
+		if (EFA_DEV_CAP(ctx, UNSOLICITED_WRITE_RECV))
+			attr->device_caps |= EFADV_DEVICE_ATTR_CAPS_UNSOLICITED_WRITE_RECV;
 	}
 
 	if (vext_field_avail(typeof(*attr), max_rdma_size, inlen)) {
@@ -449,7 +452,7 @@ static enum ibv_wc_status to_ibv_status(enum efa_io_comp_status status)
 	case EFA_IO_COMP_STATUS_FLUSHED:
 		return IBV_WC_WR_FLUSH_ERR;
 	case EFA_IO_COMP_STATUS_LOCAL_ERROR_QP_INTERNAL_ERROR:
-	case EFA_IO_COMP_STATUS_LOCAL_ERROR_INVALID_OP_TYPE:
+	case EFA_IO_COMP_STATUS_LOCAL_ERROR_UNSUPPORTED_OP:
 	case EFA_IO_COMP_STATUS_LOCAL_ERROR_INVALID_AH:
 		return IBV_WC_LOC_QP_OP_ERR;
 	case EFA_IO_COMP_STATUS_LOCAL_ERROR_INVALID_LKEY:
@@ -469,6 +472,9 @@ static enum ibv_wc_status to_ibv_status(enum efa_io_comp_status status)
 	case EFA_IO_COMP_STATUS_LOCAL_ERROR_UNRESP_REMOTE:
 		return IBV_WC_RESP_TIMEOUT_ERR;
 	case EFA_IO_COMP_STATUS_REMOTE_ERROR_BAD_ADDRESS:
+		return IBV_WC_REM_ACCESS_ERR;
+	case EFA_IO_COMP_STATUS_REMOTE_ERROR_UNKNOWN_PEER:
+		return IBV_WC_REM_OP_ERR;
 	default:
 		return IBV_WC_GENERAL_ERR;
 	}
@@ -599,6 +605,13 @@ static int efa_wc_read_sgid(struct efadv_cq *efadv_cq, union ibv_gid *sgid)
 	return 0;
 }
 
+static bool efa_wc_is_unsolicited(struct efadv_cq *efadv_cq)
+{
+	struct efa_cq *cq = efadv_cq_to_efa_cq(efadv_cq);
+
+	return EFA_GET(&cq->cur_cqe->flags, EFA_IO_CDESC_COMMON_UNSOLICITED);
+}
+
 static void efa_process_cqe(struct efa_cq *cq, struct ibv_wc *wc,
 			    struct efa_qp *qp)
 {
@@ -611,15 +624,24 @@ static void efa_process_cqe(struct efa_cq *cq, struct ibv_wc *wc,
 	wc->wc_flags = 0;
 	wc->qp_num = cqe->qp_num;
 
+	wrid_idx = cqe->req_id;
 	op_type = EFA_GET(&cqe->flags, EFA_IO_CDESC_COMMON_OP_TYPE);
-
-	if (EFA_GET(&cqe->flags, EFA_IO_CDESC_COMMON_Q_TYPE) ==
-	    EFA_IO_SEND_QUEUE) {
+	if (EFA_GET(&cqe->flags, EFA_IO_CDESC_COMMON_Q_TYPE) == EFA_IO_SEND_QUEUE) {
 		cq->cur_wq = &qp->sq.wq;
 		if (op_type == EFA_IO_RDMA_WRITE)
 			wc->opcode = IBV_WC_RDMA_WRITE;
 		else
 			wc->opcode = IBV_WC_SEND;
+
+		/* We do not have to take the WQ lock here,
+		 * because this wrid index has not been freed yet,
+		 * so there is no contention on this index.
+		 */
+		wc->wr_id = cq->cur_wq->wrid[wrid_idx];
+
+		rdma_tracepoint(rdma_core_efa, process_completion, cq->dev->name, wc->wr_id,
+				wc->status, wc->opcode, wc->qp_num, UINT32_MAX, UINT16_MAX,
+				wc->byte_len);
 	} else {
 		struct efa_io_rx_cdesc_ex *rcqe =
 			container_of(cqe, struct efa_io_rx_cdesc_ex, base.common);
@@ -643,17 +665,14 @@ static void efa_process_cqe(struct efa_cq *cq, struct ibv_wc *wc,
 			wc->imm_data = htobe32(rcqe->base.imm);
 			wc->wc_flags |= IBV_WC_WITH_IMM;
 		}
+
+		wc->wr_id = !EFA_GET(&cqe->flags, EFA_IO_CDESC_COMMON_UNSOLICITED) ?
+			cq->cur_wq->wrid[wrid_idx] : 0;
+
+		rdma_tracepoint(rdma_core_efa, process_completion, cq->dev->name, wc->wr_id,
+				wc->status, wc->opcode, wc->src_qp, wc->qp_num, wc->slid,
+				wc->byte_len);
 	}
-
-	wrid_idx = cqe->req_id;
-	/* We do not have to take the WQ lock here,
-	 * because this wrid index has not been freed yet,
-	 * so there is no contention on this index.
-	 */
-	wc->wr_id = cq->cur_wq->wrid[wrid_idx];
-
-	rdma_tracepoint(rdma_core_efa, process_completion, cq->dev->name, wc->wr_id, wc->status,
-			wc->qp_num, wc->opcode, wc->byte_len);
 }
 
 static void efa_process_ex_cqe(struct efa_cq *cq, struct efa_qp *qp)
@@ -664,19 +683,25 @@ static void efa_process_ex_cqe(struct efa_cq *cq, struct efa_qp *qp)
 
 	wrid_idx = cqe->req_id;
 
-	if (EFA_GET(&cqe->flags, EFA_IO_CDESC_COMMON_Q_TYPE) ==
-		    EFA_IO_SEND_QUEUE) {
+	if (EFA_GET(&cqe->flags, EFA_IO_CDESC_COMMON_Q_TYPE) == EFA_IO_SEND_QUEUE) {
 		cq->cur_wq = &qp->sq.wq;
+		ibvcqx->wr_id = cq->cur_wq->wrid[wrid_idx];
+		ibvcqx->status = to_ibv_status(cqe->status);
+
+		rdma_tracepoint(rdma_core_efa, process_completion, cq->dev->name, ibvcqx->wr_id,
+				ibvcqx->status, efa_wc_read_opcode(ibvcqx), cqe->qp_num,
+				UINT32_MAX, UINT16_MAX, efa_wc_read_byte_len(ibvcqx));
 	} else {
 		cq->cur_wq = &qp->rq.wq;
+		ibvcqx->wr_id = !EFA_GET(&cqe->flags, EFA_IO_CDESC_COMMON_UNSOLICITED) ?
+			cq->cur_wq->wrid[wrid_idx] : 0;
+		ibvcqx->status = to_ibv_status(cqe->status);
+
+		rdma_tracepoint(rdma_core_efa, process_completion, cq->dev->name, ibvcqx->wr_id,
+				ibvcqx->status, efa_wc_read_opcode(ibvcqx),
+				efa_wc_read_src_qp(ibvcqx), cqe->qp_num, efa_wc_read_slid(ibvcqx),
+				efa_wc_read_byte_len(ibvcqx));
 	}
-
-	ibvcqx->wr_id = cq->cur_wq->wrid[wrid_idx];
-	ibvcqx->status = to_ibv_status(cqe->status);
-
-	rdma_tracepoint(rdma_core_efa, process_completion, cq->dev->name, ibvcqx->wr_id,
-			ibvcqx->status,	qp->verbs_qp.qp.qp_num, efa_wc_read_opcode(ibvcqx),
-			efa_wc_read_byte_len(ibvcqx));
 }
 
 static inline int efa_poll_sub_cq(struct efa_cq *cq, struct efa_sub_cq *sub_cq,
@@ -712,7 +737,8 @@ static inline int efa_poll_sub_cq(struct efa_cq *cq, struct efa_sub_cq *sub_cq,
 		efa_process_ex_cqe(cq, *cur_qp);
 	} else {
 		efa_process_cqe(cq, wc, *cur_qp);
-		efa_wq_put_wrid_idx_unlocked(cq->cur_wq, cq->cur_cqe->req_id);
+		if (!EFA_GET(&cq->cur_cqe->flags, EFA_IO_CDESC_COMMON_UNSOLICITED))
+			efa_wq_put_wrid_idx_unlocked(cq->cur_wq, cq->cur_cqe->req_id);
 	}
 
 	return 0;
@@ -796,7 +822,8 @@ static int efa_next_poll(struct ibv_cq_ex *ibvcqx)
 	struct efa_cq *cq = to_efa_cq_ex(ibvcqx);
 	int ret;
 
-	efa_wq_put_wrid_idx_unlocked(cq->cur_wq, cq->cur_cqe->req_id);
+	if (!EFA_GET(&cq->cur_cqe->flags, EFA_IO_CDESC_COMMON_UNSOLICITED))
+		efa_wq_put_wrid_idx_unlocked(cq->cur_wq, cq->cur_cqe->req_id);
 	ret = efa_poll_sub_cqs(cq, NULL, true);
 
 	return ret;
@@ -807,7 +834,8 @@ static void efa_end_poll(struct ibv_cq_ex *ibvcqx)
 	struct efa_cq *cq = to_efa_cq_ex(ibvcqx);
 
 	if (cq->cur_cqe) {
-		efa_wq_put_wrid_idx_unlocked(cq->cur_wq, cq->cur_cqe->req_id);
+		if (!EFA_GET(&cq->cur_cqe->flags, EFA_IO_CDESC_COMMON_UNSOLICITED))
+			efa_wq_put_wrid_idx_unlocked(cq->cur_wq, cq->cur_cqe->req_id);
 		if (cq->db)
 			efa_update_cq_doorbell(cq, false);
 	}
@@ -846,6 +874,8 @@ static void efa_cq_fill_pfns(struct efa_cq *cq,
 
 	if (efa_attr && (efa_attr->wc_flags & EFADV_WC_EX_WITH_SGID))
 		cq->dv_cq.wc_read_sgid = efa_wc_read_sgid;
+	if (efa_attr && (efa_attr->wc_flags & EFADV_WC_EX_WITH_IS_UNSOLICITED))
+		cq->dv_cq.wc_is_unsolicited = efa_wc_is_unsolicited;
 }
 
 static void efa_sub_cq_initialize(struct efa_sub_cq *sub_cq, uint8_t *buf,
@@ -983,8 +1013,8 @@ struct ibv_cq_ex *efadv_create_cq(struct ibv_context *ibvctx,
 				  struct efadv_cq_init_attr *efa_attr,
 				  uint32_t inlen)
 {
+	uint64_t supp_wc_flags = 0;
 	struct efa_context *ctx;
-	uint64_t supp_wc_flags;
 
 	if (!is_efa_dev(ibvctx->device)) {
 		verbs_err(verbs_get_ctx(ibvctx), "Not an EFA device\n");
@@ -1001,7 +1031,10 @@ struct ibv_cq_ex *efadv_create_cq(struct ibv_context *ibvctx,
 	}
 
 	ctx = to_efa_context(ibvctx);
-	supp_wc_flags = EFA_DEV_CAP(ctx, CQ_WITH_SGID) ? EFADV_WC_EX_WITH_SGID : 0;
+	if (EFA_DEV_CAP(ctx, CQ_WITH_SGID))
+		supp_wc_flags |= EFADV_WC_EX_WITH_SGID;
+	if (EFA_DEV_CAP(ctx, UNSOLICITED_WRITE_RECV))
+		supp_wc_flags |= EFADV_WC_EX_WITH_IS_UNSOLICITED;
 	if (!check_comp_mask(efa_attr->wc_flags, supp_wc_flags)) {
 		verbs_err(verbs_get_ctx(ibvctx),
 			  "Invalid EFA wc_flags[%#lx]\n", efa_attr->wc_flags);
@@ -1312,16 +1345,20 @@ static int efa_check_qp_attr(struct efa_context *ctx,
 			     struct ibv_qp_init_attr_ex *attr,
 			     struct efadv_qp_init_attr *efa_attr)
 {
-	uint64_t supp_send_ops_mask;
 	uint64_t supp_ud_send_ops_mask = IBV_QP_EX_WITH_SEND |
-		IBV_QP_EX_WITH_SEND_WITH_IMM;
+					 IBV_QP_EX_WITH_SEND_WITH_IMM;
 	uint64_t supp_srd_send_ops_mask = IBV_QP_EX_WITH_SEND |
 					  IBV_QP_EX_WITH_SEND_WITH_IMM;
+	uint64_t supp_send_ops_mask;
+	uint16_t supp_efa_flags = 0;
+
 	if (EFA_DEV_CAP(ctx, RDMA_READ))
 		supp_srd_send_ops_mask |= IBV_QP_EX_WITH_RDMA_READ;
 	if (EFA_DEV_CAP(ctx, RDMA_WRITE))
 		supp_srd_send_ops_mask |= IBV_QP_EX_WITH_RDMA_WRITE |
 					  IBV_QP_EX_WITH_RDMA_WRITE_WITH_IMM;
+	if (EFA_DEV_CAP(ctx, UNSOLICITED_WRITE_RECV))
+		supp_efa_flags |= EFADV_QP_FLAGS_UNSOLICITED_WRITE_RECV;
 
 #define EFA_CREATE_QP_SUPP_ATTR_MASK \
 	(IBV_QP_INIT_ATTR_PD | IBV_QP_INIT_ATTR_SEND_OPS_FLAGS)
@@ -1332,6 +1369,13 @@ static int efa_check_qp_attr(struct efa_context *ctx,
 		return EOPNOTSUPP;
 	}
 
+	if (!check_comp_mask(efa_attr->flags, supp_efa_flags)) {
+		verbs_err(&ctx->ibvctx,
+			  "Unsupported EFA flags[%#x] supported[%#x]\n",
+			  efa_attr->flags, supp_efa_flags);
+		return EOPNOTSUPP;
+	}
+
 	if (!check_comp_mask(attr->comp_mask, EFA_CREATE_QP_SUPP_ATTR_MASK)) {
 		verbs_err(&ctx->ibvctx,
 			  "Unsupported comp_mask[%#x] supported[%#x]\n",
@@ -1453,6 +1497,8 @@ static struct ibv_qp *create_qp(struct ibv_context *ibvctx,
 		sizeof(struct efa_io_tx_wqe);
 	if (attr->qp_type == IBV_QPT_DRIVER)
 		req.driver_qp_type = efa_attr->driver_qp_type;
+	if (efa_attr->flags & EFADV_QP_FLAGS_UNSOLICITED_WRITE_RECV)
+		req.flags |= EFA_CREATE_QP_WITH_UNSOLICITED_WRITE_RECV;
 
 	err = ibv_cmd_create_qp_ex(ibvctx, &qp->verbs_qp,
 				   attr, &req.ibv_cmd, sizeof(req),
@@ -1510,6 +1556,7 @@ struct ibv_qp *efa_create_qp(struct ibv_pd *ibvpd,
 			     struct ibv_qp_init_attr *attr)
 {
 	struct ibv_qp_init_attr_ex attr_ex = {};
+	struct efadv_qp_init_attr efa_attr = {};
 	struct ibv_qp *ibvqp;
 
 	if (attr->qp_type != IBV_QPT_UD) {
@@ -1523,7 +1570,7 @@ struct ibv_qp *efa_create_qp(struct ibv_pd *ibvpd,
 	attr_ex.comp_mask = IBV_QP_INIT_ATTR_PD;
 	attr_ex.pd = ibvpd;
 
-	ibvqp = create_qp(ibvpd->context, &attr_ex, NULL);
+	ibvqp = create_qp(ibvpd->context, &attr_ex, &efa_attr);
 	if (ibvqp)
 		memcpy(attr, &attr_ex, sizeof(*attr));
 
@@ -1533,13 +1580,15 @@ struct ibv_qp *efa_create_qp(struct ibv_pd *ibvpd,
 struct ibv_qp *efa_create_qp_ex(struct ibv_context *ibvctx,
 				struct ibv_qp_init_attr_ex *attr_ex)
 {
+	struct efadv_qp_init_attr efa_attr = {};
+
 	if (attr_ex->qp_type != IBV_QPT_UD) {
 		verbs_err(verbs_get_ctx(ibvctx), "Unsupported QP type\n");
 		errno = EOPNOTSUPP;
 		return NULL;
 	}
 
-	return create_qp(ibvctx, attr_ex, NULL);
+	return create_qp(ibvctx, attr_ex, &efa_attr);
 }
 
 struct ibv_qp *efadv_create_driver_qp(struct ibv_pd *ibvpd,
@@ -1580,6 +1629,8 @@ struct ibv_qp *efadv_create_qp_ex(struct ibv_context *ibvctx,
 				  struct efadv_qp_init_attr *efa_attr,
 				  uint32_t inlen)
 {
+	struct efadv_qp_init_attr local_efa_attr = {};
+
 	if (!is_efa_dev(ibvctx->device)) {
 		verbs_err(verbs_get_ctx(ibvctx), "Not an EFA device\n");
 		errno = EOPNOTSUPP;
@@ -1597,7 +1648,8 @@ struct ibv_qp *efadv_create_qp_ex(struct ibv_context *ibvctx,
 		return NULL;
 	}
 
-	return create_qp(ibvctx, attr_ex, efa_attr);
+	memcpy(&local_efa_attr, efa_attr, min_t(uint32_t, inlen, sizeof(local_efa_attr)));
+	return create_qp(ibvctx, attr_ex, &local_efa_attr);
 }
 
 int efa_modify_qp(struct ibv_qp *ibvqp, struct ibv_qp_attr *attr,
