@@ -163,6 +163,7 @@ class RDMATestCase(unittest.TestCase):
         self.pre_environment = {}
         self.server = None
         self.client = None
+        self.iters = 10
 
 
     def is_eth_and_has_roce_hw_bug(self):
@@ -180,10 +181,21 @@ class RDMATestCase(unittest.TestCase):
             has_roce_hw_bug(vendor_id, vendor_pid)
 
     @staticmethod
-    def get_net_name(dev):
-        out = subprocess.check_output(['ls',
-                                       '/sys/class/infiniband/{}/device/net/'
-                                      .format(dev)])
+    def get_net_name(dev, port=None):
+        if port is not None:
+            out = subprocess.check_output(['rdma', 'link',  'show', '-j'])
+            loaded_json = json.loads(out.decode())
+            for row in loaded_json:
+                try:
+                    if row['ifname'] == dev and row['port'] == port:
+                        return row['netdev']
+                except KeyError:
+                    pass
+
+        if not os.path.exists(f'/sys/class/infiniband/{dev}/device/net/'):
+            return None
+
+        out = subprocess.check_output(['ls', f'/sys/class/infiniband/{dev}/device/net/'])
         return out.decode().split('\n')[0]
 
     @staticmethod
@@ -265,10 +277,10 @@ class RDMATestCase(unittest.TestCase):
         self._add_gids_per_port(ctx, dev, self.ib_port)
 
     def _get_ip_mac(self, dev, port, idx):
-        if not os.path.exists('/sys/class/infiniband/{}/device/net/'.format(dev)):
+        net_name = self.get_net_name(dev, port)
+        if net_name is None:
             self.args.append([dev, port, idx, None, None])
             return
-        net_name = self.get_net_name(dev)
         try:
             ip_addr, mac_addr = self.get_ip_mac_address(net_name)
         except (KeyError, IndexError):
@@ -302,6 +314,45 @@ class RDMATestCase(unittest.TestCase):
             self.pre_environment[var] = os.environ.get(var)
         os.environ[var] = value
 
+    def sync_remote_attr(self):
+        """
+        Sync the MR remote attributes between the server and the client.
+        """
+        self.server.rkey = self.client.mr.rkey
+        self.server.raddr = self.client.mr.buf
+        self.client.rkey = self.server.mr.rkey
+        self.client.raddr = self.server.mr.buf
+
+    def pre_run(self):
+        """
+        Configure Resources before running traffic.
+        pre_run() must be implemented by the client and server.
+        """
+        self.client.pre_run(self.server.psns, self.server.qps_num)
+        self.server.pre_run(self.client.psns, self.client.qps_num)
+
+    def create_players(self, resource, sync_attrs=True, **resource_arg):
+        """
+        Init test resources.
+        :param resource: The RDMA resources to use.
+        :param sync_attrs: If True, sync remote attrs such as rkey and raddr
+        :param resource_arg: Dict of args that specify the resource specific
+                             attributes.
+        """
+        try:
+            self.client = resource(**self.dev_info, **resource_arg)
+            self.server = resource(**self.dev_info, **resource_arg)
+        except PyverbsRDMAError as ex:
+            if ex.error_code == errno.EOPNOTSUPP:
+                raise unittest.SkipTest(f'Create player of {resource.__name__} is not supported')
+            raise ex
+        self.pre_run()
+        if sync_attrs:
+            self.sync_remote_attr()
+        self.traffic_args = {'client': self.client, 'server': self.server,
+                             'iters': self.iters, 'gid_idx': self.gid_index,
+                             'port': self.ib_port}
+
     def tearDown(self):
         """
         Restore the previous environment variables values before ending the test.
@@ -627,12 +678,19 @@ class TrafficResources(BaseResources):
 
     def pre_run(self, rpsns, rqps_num):
         """
-        Modify the QP's states to RTS and fill receive queue with <num_msgs> work
-        requests.
-        This method is not implemented in this class.
-        :param rpsns: Remote PSNs
+        Configure resources before running traffic and modifies the QP to RTS
+        if required.
+        :param rpsns: Remote PSNs (packet serial numbers)
         :param rqps_num: Remote QPs Number
-        :return: None
+        """
+        self.rpsns = rpsns
+        self.rqps_num = rqps_num
+        self.to_rts()
+
+    def to_rts(self):
+        """
+        Modify the QP's states to RTS and initialize it to be ready for traffic.
+        If not required, can be "passed" but must be implemented.
         """
         raise NotImplementedError()
 
@@ -667,17 +725,6 @@ class RCResources(RoCETrafficResources):
             attr.sq_psn = self.rpsns[i]
             self.qps[i].to_rts(attr)
 
-    def pre_run(self, rpsns, rqps_num):
-        """
-        Configure Resources before running traffic
-        :param rpsns: Remote PSNs (packet serial number)
-        :param rqps_num: Remote QPs number
-        :return: None
-        """
-        self.rpsns = rpsns
-        self.rqps_num = rqps_num
-        self.to_rts()
-
 
 class UDResources(RoCETrafficResources):
     UD_QKEY = 0x11111111
@@ -708,9 +755,8 @@ class UDResources(RoCETrafficResources):
                     raise unittest.SkipTest(f'Create QP type {qp_init_attr.qp_type} is not supported')
                 raise ex
 
-    def pre_run(self, rpsns, rqps_num):
-        self.rpsns = rpsns
-        self.rqps_num = rqps_num
+    def to_rts(self):
+        pass
 
 
 class RawResources(TrafficResources):
@@ -718,6 +764,9 @@ class RawResources(TrafficResources):
         return QPInitAttr(qp_type=e.IBV_QPT_RAW_PACKET, scq=self.cq,
                           rcq=self.cq, srq=self.srq, cap=self.create_qp_cap())
 
+    def pre_run(self, rpsns=None, rqps_num=None):
+        pass
+
 
 class XRCResources(RoCETrafficResources):
     def __init__(self, dev_name, ib_port, gid_index, qp_count=2):
@@ -733,6 +782,10 @@ class XRCResources(RoCETrafficResources):
         os.close(self.xrcd_fd)
         self.temp_file.close()
 
+    @property
+    def qp(self):
+        return self.sqp_lst[0]
+
     def create_qps(self):
         """
         Initializes self.qps with an XRC SEND/RECV QPs.
@@ -818,8 +871,3 @@ class XRCResources(RoCETrafficResources):
         self.create_xrcd()
         super(XRCResources, self).init_resources()
         self.create_srq()
-
-    def pre_run(self, rpsns, rqps_num):
-        self.rqps_num = rqps_num
-        self.rpsns = rpsns
-        self.to_rts()
